# AUTOGENERATED! DO NOT EDIT! File to edit: 02_chars.ipynb (unless otherwise specified).

__all__ = ['chardatadir', 'charname', 'charcategory', 'charsubcategory', 'charblock', 'blockfamily', 'unicodeblocks',
           'unicodecategories', 'unicodefamilies', 'NormChange', 'NormResult', 'TextNormalizer']

# Cell
import pandas as pd
pd.options.display.max_rows = 100
pd.options.display.max_columns = 50

# Cell
from .core import *

# Cell
chardatadir = config.libdata / "chars"

# Cell
from unicodedata import name as unicodename
from unicodedata import category as unicodecategory

unicodeblocks = pd.read_csv(chardatadir / "unicode_blocks.csv", sep=";")
unicodeblocks["StartNum"] = unicodeblocks["Start"].map(lambda hex:int(hex,16))
unicodeblocks["EndNum"] = unicodeblocks["End"].map(lambda hex:int(hex,16))

unicodecategories = pd.read_csv(chardatadir / "unicode_categories.csv", sep=";")

unicodefamilies = pd.read_csv(chardatadir / "unicode_families.csv", sep=";")

def charname(char):
    return unicodename(char,f"Char {ord(char)}").title()

def charcategory(char):
    catcode = unicodecategory(char)
    catrow = unicodecategories.loc[unicodecategories["Code"]==catcode]
    return catrow["Category"].iloc[0]

def charsubcategory(char):
    catcode = unicodecategory(char)
    catrow = unicodecategories.loc[unicodecategories["Code"]==catcode]
    return catrow["Subcategory"].iloc[0]

def charblock(char):
    charnum = ord(char)
    blockrow = unicodeblocks[(unicodeblocks["StartNum"]<= charnum) & (unicodeblocks["EndNum"]>= charnum)]
    return blockrow["Block"].iloc[0]

def blockfamily(block):
    filteredseries = unicodefamilies[unicodefamilies["UnicodeBlock"]==block]
    if len(filteredseries) == 0:
        family= "Other"
    else:
        family = filteredseries["CharFamily"].iloc[0]
    return family

# Cell
class NormChange():

    def __init__(self, layer, index, charsInput, charsOutput, removedInfo=None):
        self.layer, self.index, self.input, self.output, self.removedInfo = layer, index, charsInput, charsOutput, removedInfo

    def __repr__(self):
        return f"{self.layer} - {self.index} : {self.input} => {self.output}"

# Cell
class NormResult():

    def __init__(self, inputText, transformsDescs):
        self.input, self.transforms = inputText, transformsDescs
        self.layerChanges = None
        self.output = ""

    def addChange(self, layer, index, charsInput, charsOutput, removedInfo=None):
        if self.layerChanges is None:
            self.layerChanges = []
        if layer > (len(self.layerChanges)-1):
            for i in range(0,layer-len(self.layerChanges)+1):
                self.layerChanges.append([])
        changes = self.layerChanges[layer]
        change = NormChange(layer,index,charsInput,charsOutput,removedInfo)
        changes.append(change)

    def describeChanges(self):
         if self.layerChanges is None:
            return 'No change'
         else:
            desc = StringIO()
            previousString = self.input
            for changes in self.layerChanges:
                if len(changes) == 0: continue
                layer = changes[0].layer
                layerDesc = self.transforms[layer]
                desc.write(layerDesc+"\n")
                dispInparts = []
                outparts = []
                dispOutparts = []
                lastIndex = 0
                for change in changes:
                    if change.index > lastIndex:
                        samePart = previousString[lastIndex:change.index]
                        dispInparts.append(samePart)
                        outparts.append(samePart)
                        dispOutparts.append(samePart)
                    dispInpart = change.input
                    outpart = change.output
                    dispOutpart = outpart
                    if len(dispInpart)>len(outpart):
                        dispOutpart = outpart + ("_"*(len(dispInpart)-len(outpart)))
                    elif len(outpart)>len(dispInpart):
                        dispInpart = dispInpart + (" "*(len(outpart)-len(dispInpart)))
                    dispInparts.append(' ['+dispInpart+'] ')
                    outparts.append(outpart)
                    dispOutparts.append(' ['+dispOutpart+'] ')
                    lastIndex = change.index + len(change.input)
                if lastIndex < len(previousString):
                    samePart = previousString[lastIndex:]
                    dispInparts.append(samePart)
                    outparts.append(samePart)
                    dispOutparts.append(samePart)
                previousString = "".join(outparts)
                desc.write(" < ")
                for inpart in dispInparts:
                    desc.write(inpart)
                desc.write('\n')
                desc.write(" < ")
                for outpart in dispOutparts:
                    desc.write(outpart)
                desc.write('\n')
            return desc.getvalue()

    def mapOutputIndexToInput(self,outputIndex):
        inputIndex = outputIndex
        for changes in self.layerChanges:
            outputIndex = inputIndex
            for change in changes:
                if outputIndex < change.index:
                    break
                elif outputIndex > (change.index + len(change.output)):
                    inputIndex = inputIndex + (len(change.input)-len(change.output))
                else:
                    inputIndex = inputIndex -(outputIndex-change.index)
                    break
        return inputIndex

    def setOutput(self, outputText):
        self.output = outputText

    def __repr__(self):
        return self.output

# Cell
from functools import partial
from operator import itemgetter
from io import StringIO

class TextNormalizer():

    def __init__(self):

        # 1. Load Unicode character set data for latin script
        # 1.1 Frequent encoding errors : windows1252 read as iso8859-1
        dfencodingwin1252 = pd.read_csv(chardatadir / "windows1252-iso8859-errors.csv", sep=";")
        win1252errorchars = {}
        for rowidx,row in dfencodingwin1252.iterrows():
            win1252errorchars[row["Char"]] = row["DecodedChar"]
        # 1.2 Frequent encoding errors : utf8 read as windows1252
        dfencodingutf8 = pd.read_csv(chardatadir / "utf8-windows1252-errors.csv", sep=";")
        utf8errorchars = {}
        for rowidx,row in dfencodingutf8.iterrows():
            utf8errorchars[row["ErrorSubstring"]] = row["DecodedChar"]
        utf8errorshdict = self.buildhierarchicaldict(utf8errorchars)
        # 1.3 Frequent encoding errors : windows1252 read as utf8
        dfencodingwin1252utf8 = pd.read_csv(chardatadir / "windows1252-utf8-errors.csv", sep=";")
        win1252utf8errorchars = {}
        for rowidx,row in dfencodingwin1252utf8.iterrows():
            win1252utf8errorchars[row["Char"]] = row["DecodedChars"]
        # 1.4 Unicode combining chars
        dfcombiningchars = pd.read_csv(chardatadir / "combiningdiacritics.csv", sep=";")
        combiningchars = {}
        for rowidx,row in dfcombiningchars.iterrows():
            combiningchars[row["BaseChar"]+row["Char"]] = row["CombinedChar"]
        combiningcharshdict = self.buildhierarchicaldict(combiningchars)
        # 1.5 Control chars
        dfcontrolchars = pd.read_csv(chardatadir / "controlchars.csv", sep=";")
        dfcontrolchars.loc[0,"Char"] = chr(0) # chr(0) can't be saved in CSV file
        controlchars = set(dfcontrolchars["Char"])
        # 1.6 Latin letter symbols
        dflatinsymbols = pd.read_csv(chardatadir / "latinsymbols.csv", sep=";")
        latinlettersnolayout = {}
        latinlettersremovedlayout = {}
        for rowidx,row in dflatinsymbols.iterrows():
            latinlettersnolayout[row["Char"]] = row["NormString"]
            latinlettersremovedlayout[row["Char"]] = row["Layout"]
        # 1.7 Latin letters
        dflatinletters = pd.read_csv(chardatadir / "latinletters.csv", sep=";")
        latinletterstoupper = {}
        for rowidx,row in dflatinletters.iterrows():
            if row["Char"] != row["UpperChar"]:
                latinletterstoupper[row["Char"]] = row["UpperChar"]
        latinlettersnodiacritics = {}
        latinlettersremoveddiacritics = {}
        for rowidx,row in dflatinletters.iterrows():
            if row["IsDiacritic"]:
                latinlettersnodiacritics[row["Char"]] = row["BaseChar"]
                latinlettersremoveddiacritics[row["Char"]] = row["Diacritics"]
        latinlettersnoligatures = {}
        for rowidx,row in dflatinletters.iterrows():
            if row["IsLigature"]:
                latinlettersnoligatures[row["Char"]] = row["MultiChars"]
        # 1.8 Latin numbers and number symbols
        dflatinnumbers = pd.read_csv(chardatadir / "latinnumbers.csv", sep=";")
        latinnumbersnolayout = {}
        latinnumbersremovedlayout = {}
        for rowidx,row in dflatinnumbers.iterrows():
            if rowidx < 10:
                continue
            latinnumbersnolayout[row["Char"]] = row["NormString"]
            latinnumbersremovedlayout[row["Char"]] = row["Layout"]
        # 1.9 Variations on frequent chars to normalize
        dfnormchars = pd.read_csv(chardatadir / "normalizedchars.csv", sep=";")
        normalizedchars = {}
        for rowidx,row in dfnormchars.iterrows():
            normalizedchars[row["Char"]] = row["NormChar"]
        # 1.10 Optional replacement of cyrillic and greek chars looking like latin letters
        dfcgnormchars = pd.read_csv(chardatadir / "cyrillic-greek-chars.csv", sep=";")
        cgnormalizedchars = {}
        for rowidx,row in dfcgnormchars.iterrows():
            cgnormalizedchars[row["Char"]] = row["NormChar"]
        # 1.11 Final supported french charset
        dfsupportedchars = pd.read_csv(chardatadir / "charset-fr.csv", sep=";", quotechar='"')
        dfsupportedchars.loc[0,"Char"] = chr(0) # chr(0) can't be saved in CSV file
        supportedchars = set(dfsupportedchars["Char"])

        # 2.1 List successive transformations
        self.transformsDescs = []
        transforms = []
        self.transformsDescs.append("Fix encoding errors : windows1252 read as iso8859-1")
        transforms.append(partial(self.replacechars1to1, 0, win1252errorchars))
        self.transformsDescs.append("Fix encoding errors : utf8 read as windows1252")
        transforms.append(partial(self.replacecharsNto1, 1, utf8errorshdict))
        self.transformsDescs.append("Fix encoding errors :  windows1252 read as utf8")
        transforms.append(partial(self.replacechars1toN, 2, win1252utf8errorchars))
        self.transformsDescs.append("Merge Unicode combining chars")
        transforms.append(partial(self.replacecharsNto1, 3, combiningcharshdict))
        self.transformsDescs.append("Ignore control chars")
        transforms.append(partial(self.ignorechars, 4, controlchars))
        self.transformsDescs.append("Replace latin letter symbols")
        transforms.append(partial(self.replacechars1toN, 5, latinlettersnolayout))
        self.transformsDescs.append("Replace latin letter ligatures")
        transforms.append(partial(self.replacechars1toN, 6, latinlettersnoligatures))
        self.transformsDescs.append("Replace latin number symbols")
        transforms.append(partial(self.replacechars1toN, 7, latinnumbersnolayout))
        self.transformsDescs.append("Normalize equivalent chars")
        transforms.append(partial(self.replacechars1to1, 8, normalizedchars))
        self.transformsDescs.append("Replace cyrillic and greek chars looking like latin letters")
        transforms.append(partial(self.replacechars1to1,9, cgnormalizedchars))
        self.transformsDescs.append("Replace infrequent chars : latin letters with diacritics")
        transforms.append(partial(self.replacecharsnotinset, 10, supportedchars, latinlettersnodiacritics))
        self.transformsDescs.append("Replace infrequent chars : other scripts")
        transforms.append(partial(self.replaceotherscripts, 11, supportedchars))
        self.transformsDescs.append("Replace infrequent chars : symbols")
        transforms.append(partial(self.replacesymbols, 12, supportedchars))
        self.transformsDescs.append("Replace infrequent chars : chars to ignore")
        transforms.append(partial(self.ignoreotherchars, 13, supportedchars))

        # 2.2 Combine all transformations
        def func(x,y):
            ci = transforms[0](x,y)
            for transform in transforms[1:]:
                ci = transform(ci,y)
            return ci
        self.transformsFunc = func

    def __repr__(self):
        desc = StringIO()
        for idx,transformDesc in enumerate(self.transformsDescs):
            desc.write(f'{idx+1} - {transformDesc}\n')
        return desc.getvalue()

    def __call__(self, inputText):
        result = NormResult(inputText, self.transformsDescs)
        result.setOutput(self.tostring(self.transformsFunc(inputText,result)))
        return result

    @staticmethod
    def buildhierarchicaldict(idict):
        hdict = {}
        odicts = []
        for key in idict:
            if len(key) > 1:
                firstchar = key[0]
                remainingstring = key[1:]
                if not firstchar in hdict:
                    newdict = {}
                    hdict[firstchar] = newdict
                    odicts.append((firstchar,newdict))
                hdict[firstchar][remainingstring] = idict[key]
        for pkey,odict in odicts:
            dictwithlongkey = False
            for key in odict:
                if len(key)>1:
                    dictwithlongkey = True
                    break
            if dictwithlongkey:
                hdict[pkey] = TextNormalizer.buildhierarchicaldict(odict)
        return hdict

    @staticmethod
    def ignorechars(layer, charset, chariterator, result):
        for index,char in enumerate(chariterator):
            if not char in charset:
                yield char
            else:
                result.addChange(layer, index, char, '')

    @staticmethod
    def replacechars1to1(layer, chardict, chariterator, result):
        for index,char in enumerate(chariterator):
            if char in chardict:
                resChar = chardict[char]
                result.addChange(layer, index, char, resChar)
                yield resChar
            else:
                yield char

    @staticmethod
    def replacechars1toN(layer, chardict, chariterator, result):
        for index,char in enumerate(chariterator):
            if char in chardict:
                resStr = chardict[char]
                result.addChange(layer, index, char, resStr)
                for outchar in resStr:
                    yield outchar
            else:
                yield char

    @staticmethod
    def replacecharsNto1(layer, hierarchicaldict, chariterator, result):
        candidatechars = []
        candidatedicts = []
        for index,char in enumerate(chariterator):
            # Try to match previously started patterns
            if len(candidatechars)>0:
                for idx,candidatedict in enumerate(candidatedicts):
                    if not candidatedict is None:
                        if char in candidatedict:
                            value = candidatedict[char]
                            if isinstance(value,dict):
                                candidatedicts[idx] = value
                            else:
                                # Success : found a char to return
                                for ridx in range(0,idx):
                                    yield candidatechars[ridx]
                                replacedStr = "".join(candidatechars[idx:]) + char
                                result.addChange(layer, index-len(replacedStr)+1, replacedStr, value)
                                candidatechars = []
                                candidatedicts = []
                                char = None
                                yield value
                                break
                        else:
                            candidatedicts[idx] = None
                # Clean oldest failed attemps and return accumulated chars
                while len(candidatedicts)>0 and candidatedicts[0] is None:
                    candidatedicts.pop(0)
                    yield candidatechars.pop(0)
            # Handle the current char
            if not char is None:
                if len(candidatechars)==0:
                    if char in hierarchicaldict:
                        value = hierarchicaldict[char]
                        if isinstance(value,dict):
                            candidatechars.append(char)
                            candidatedicts.append(value)
                        else:
                            result.addChange(layer, index, char, value)
                            yield value
                    else:
                        yield char
                else:
                    candidatechars.append(char)
                    if char in hierarchicaldict:
                        value = hierarchicaldict[char]
                        candidatedicts.append(value)
                    else:
                        candidatedicts.append(None)
        if len(candidatechars)>0:
            for char in candidatechars:
                yield char

    @staticmethod
    def replacecharsnotinset(layer, charset, replacedict, chariterator, result):
        for index,char in enumerate(chariterator):
            if char in charset:
                yield char
            else:
                if char in replacedict:
                    resChar = replacedict[char]
                    result.addChange(layer, index, char, resChar)
                    yield resChar
                else:
                    yield char

    @staticmethod
    def replaceotherscripts(layer, charset, chariterator, result):
        for index,char in enumerate(chariterator):
            if char in charset:
                yield char
            else:
                family = blockfamily(charblock(char))
                if not family in ("Symbols","Ignore"):
                    resStr = chr(65532) + str(ord(char)) + '_'
                    result.addChange(layer, index, char, resStr)
                    for outchar in resStr:
                        yield outchar
                else:
                    yield char

    @staticmethod
    def replacesymbols(layer, charset, chariterator, result):
        for index,char in enumerate(chariterator):
            if char in charset:
                yield char
            else:
                family = blockfamily(charblock(char))
                if family == "Symbols":
                    resStr ='$' + charname(char).replace(' ','') + '_'
                    result.addChange(layer, index, char, resStr)
                    for outchar in resStr:
                        yield outchar
                else:
                    yield char

    @staticmethod
    def ignoreotherchars(layer, charset, chariterator, result):
        for index,char in enumerate(chariterator):
            if char in charset:
                yield char
            else:
                family = blockfamily(charblock(char))
                if family == "Ignore":
                    result.addChange(layer, index, char, '')
                else:
                    yield char

    @staticmethod
    def tostring(iterator):
        return "".join(iterator)


# AUTOGENERATED! DO NOT EDIT! File to edit: 00_core.ipynb (unless otherwise specified).

__all__ = ['Config', 'config', 'download_url']

# Cell
import os
from pathlib import Path
from zipfile import ZipFile,ZIP_LZMA

# Cell
class Config:
    config_path = Path(os.getenv('FRENCHTEXT_HOME', '~/.frenchtext')).expanduser()

    def __init__(self):
        self.config_path.mkdir(parents=True, exist_ok=True)
        self.create_config()
        for key in self.d:
            Path(self.d[key]).mkdir(parents=True, exist_ok=True)

    def __getitem__(self,k):
        k = k.lower()
        if k not in self.d: k = k+'_path'
        return Path(self.d[k])

    def __getattr__(self,k):
        if k=='d': raise AttributeError
        return self[k]

    def __setitem__(self,k,v): self.d[k] = str(v)
    def __contains__(self,k): return k in self.d

    def create_config(self):
        self.d = {'datasets_path':    str(self.config_path/'datasets'),
                  'models_path':      str(self.config_path/'models')}

        try:
            self.d["libdata_path"] = str(Path(__file__).parent/"data")
        except NameError:
            self.d["libdata_path"] = str(Path(os.getcwd())/"frenchtext"/"data")

# Cell
config = Config()

# Cell
import requests
from fastprogress.fastprogress import progress_bar,master_bar

# Cell
def download_url(url, dest, file_size=0, overwrite=False, pbar=None, show_progress=True, chunk_size=1024*1024,
                 timeout=10, retries=3):
    "Download `url` to `dest` unless it exists and not `overwrite`"
    if os.path.exists(dest) and not overwrite: return

    s = requests.Session()
    s.mount('http://',requests.adapters.HTTPAdapter(max_retries=retries))
    # additional line to identify as a firefox browser, see fastai/#2438
    s.headers.update({'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:71.0) Gecko/20100101 Firefox/71.0'})
    u = s.get(url, stream=True, timeout=timeout)
    if file_size==0:
        try: file_size = int(u.headers["Content-Length"])
        except: show_progress = False

    with open(dest, 'wb') as f:
        nbytes = 0
        if show_progress:
            pbar = progress_bar(range(file_size), leave=False, parent=pbar)
            pbar.comment = dest.name
        try:
            if show_progress: pbar.update(0)
            for chunk in u.iter_content(chunk_size=chunk_size):
                nbytes += len(chunk)
                if show_progress: pbar.update(nbytes)
                f.write(chunk)
        except requests.exceptions.ConnectionError as e:
            fname = url.split('/')[-1]
            data_dir = dest.parent
            print(f'\n Download of {url} has failed after {retries} retries\n'
                  f' Fix the download manually:\n'
                  f'$ mkdir -p {data_dir}\n'
                  f'$ cd {data_dir}\n'
                  f'$ wget -c {url}\n'
                  f'$ tar xf {fname}\n'
                  f' And re-run your code once the download is successful\n')
            return

    if dest.name.endswith(".zip"):
        file = ZipFile(dest, compression=ZIP_LZMA)
        print(f"Extracting {dest.name} (this may last several seconds) ...")
        file.extractall(dest.parent)
        file.close()
        os.remove(dest)
        print("OK")


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Config": "00_core.ipynb",
         "config": "00_core.ipynb",
         "download_url": "00_core.ipynb",
         "pd.options.display.max_rows": "02_chars.ipynb",
         "pd.options.display.max_columns": "02_chars.ipynb",
         "list_datasets": "01_datasets.ipynb",
         "datasetsdf": "01_datasets.ipynb",
         "list_dataset_files": "01_datasets.ipynb",
         "read_download_info": "01_datasets.ipynb",
         "download_all_datasets": "01_datasets.ipynb",
         "download_dataset_file": "01_datasets.ipynb",
         "downloaddf": "01_datasets.ipynb",
         "read_dataset_file": "01_datasets.ipynb",
         "download_urls_file": "01_datasets.ipynb",
         "read_urls_file": "01_datasets.ipynb",
         "urlsdf": "01_datasets.ipynb",
         "get_rows_from_datasetdf": "01_datasets.ipynb",
         "show_first_rows": "01_datasets.ipynb",
         "get_textblocks_from_dataset": "01_datasets.ipynb",
         "show_first_textblocks": "01_datasets.ipynb",
         "get_text_from_rowindex": "01_datasets.ipynb",
         "get_url_from_rowindex": "01_datasets.ipynb",
         "find_textblocks_with_chars": "01_datasets.ipynb",
         "getContextAroundWord": "01_datasets.ipynb",
         "chardatadir": "02_chars.ipynb",
         "charname": "02_chars.ipynb",
         "charcategory": "02_chars.ipynb",
         "charsubcategory": "02_chars.ipynb",
         "charblock": "02_chars.ipynb",
         "blockfamily": "02_chars.ipynb",
         "unicodeblocks": "02_chars.ipynb",
         "unicodecategories": "02_chars.ipynb",
         "unicodefamilies": "02_chars.ipynb",
         "NormChange": "02_chars.ipynb",
         "NormResult": "02_chars.ipynb",
         "TextNormalizer": "02_chars.ipynb"}

modules = ["core.py",
           "datasets.py",
           "chars.py"]

doc_url = "https://laurentprudhon.github.io/frenchtext/"

git_url = "https://github.com/laurentprudhon/frenchtext/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: 01_datasets.ipynb (unless otherwise specified).

__all__ = ['list_datasets', 'datasetsdf', 'list_dataset_files', 'read_download_info', 'download_all_datasets',
           'download_dataset_file', 'downloaddf', 'read_dataset_file', 'download_urls_file', 'read_urls_file', 'urlsdf',
           'get_rows_from_datasetdf', 'show_first_rows', 'get_textblocks_from_dataset', 'show_first_textblocks',
           'get_text_from_rowindex', 'get_url_from_rowindex', 'find_textblocks_with_chars', 'getContextAroundWord']

# Cell
import numpy as np
import pandas as pd
pd.options.display.max_rows = 100
pd.options.display.max_columns = 50

# Cell
from .core import *

# Cell
def list_datasets():
    return pd.read_csv(config.libdata /"datasets"/"datasets.csv",sep=';')

datasetsdf = list_datasets()

# Cell
def list_dataset_files():
    return datasetsdf["DatasetFile"].unique().tolist()

# Cell
def read_download_info():
    return pd.read_csv(config.libdata /"datasets"/"download_info.csv",sep=';')

downloaddf = read_download_info()

def download_all_datasets():
    for datasetfile in datasetsdf['DatasetFile'].unique():
        download_dataset_file(datasetfile)

import math

def download_dataset_file(datasetfile):
    row = downloaddf[downloaddf["DatasetFile"]==datasetfile].iloc[0]
    url = row["DownloadUrl"]
    size = row["FileSize"]
    print(f"Downloading dataset file : {datasetfile} ({math.floor(size/1024/1024)} MB)")
    resfile = config.datasets / (datasetfile+".dataset.feather")
    if not resfile.exists():
        download_url(url, config.datasets / (datasetfile+".dataset.zip"), size)

# Cell
def read_dataset_file(datasetfile):
    datasetdffile = config.datasets / (datasetfile+".dataset.feather")
    if(datasetdffile.exists()):
        datasetdf = pd.read_feather(datasetdffile)
        print(f"Loaded dataframe for dataset {datasetfile} : {len(datasetdf)} text blocks")
        return datasetdf
    else:
        raise Exception(f"No dataframe for dataset {datasetfile}")

# Cell
def download_urls_file():
    row = downloaddf[downloaddf["DatasetFile"]=="urls"].iloc[0]
    url = row["DownloadUrl"]
    size = row["FileSize"]
    print(f"Downloading datasets urls file ({math.floor(size/1024/1024)} MB)")
    download_url(url, config.datasets / "datasets.urls.zip", size)
    print("Done")

urlsdf = None

def read_urls_file():
    global urlsdf
    if urlsdf is None:
        download_urls_file()
        urlsdf = pd.read_feather(config.datasets / "datasets.urls.feather")
    print(f"Loaded datasets urls : {len(urlsdf)} urls")
    return urlsdf

# Cell
def get_rows_from_datasetdf(datasetdf, minwords=5, maxwords=None, lang=None):
    condition = datasetdf.index>=0
    if isinstance(minwords,int):
        condition = condition & (datasetdf["Words"] >= minwords)
    if isinstance(maxwords,int):
        condition = condition & (datasetdf["Words"] <= maxwords)
    if isinstance(lang,str):
        condition = condition & (datasetdf["Lang"] == lang)
    return datasetdf[condition].iterrows()

def show_first_rows(rowsiterator, count=5, skip=0):
    i=0
    for rowidx,row in rowsiterator:
        i = i+1
        if(i <= skip): continue
        text = row["Text"]
        print(f"{rowidx} - {text}")
        if(i >= skip+count): break

# Cell
def get_textblocks_from_dataset(dataset, minwords=5, maxwords=None, lang=None):
    for datasetfile in datasetsdf.loc[datasetsdf["Dataset"]==dataset,"DatasetFile"].unique():
        datasetdf = read_dataset_file(datasetfile)
        for rowidx,row in get_rows_from_datasetdf(datasetdf,minwords,maxwords,lang):
            yield row["Text"]

def show_first_textblocks(textiterator, count=5, skip=0):
    i=0
    for text in textiterator:
        i = i+1
        if(i <= skip): continue
        print(f"{i} - {text}")
        if(i >= (skip+count)): break

# Cell
def get_text_from_rowindex(datasetdf, rowidx):
    return datasetdf.iloc[rowidx]["Text"]

def get_url_from_rowindex(datasetdf, rowidx):
    global urlsdf
    row = datasetdf.iloc[rowidx]
    website = row["Website"]
    docid = row["DocId"]
    if urlsdf is None:
        urlsdf = read_urls_file()
    url = urlsdf.loc[(urlsdf["Website"] == website) & (urlsdf["DocId"] == docid),"DocUrl"].values[0]
    return url

# Cell
def find_textblocks_with_chars(datasetdf, chars, count=100, ctxsize=20, wrap=False):
    if isinstance(chars,int): chars = chr(chars)
    textsWithWord = datasetdf[datasetdf["Text"].str.contains(chars,regex=False)]
    if count>0 and (textsWithWord["Text"].count()>count):
        textsWithWord = textsWithWord.sample(count)
    return textsWithWord["Text"].apply(lambda t: getContextAroundWord(t,chars,ctxsize,wrap))

def getContextAroundWord(text,word,ctxsize=20,wrap=False):
    idx = text.index(word)
    start = max(idx-ctxsize,0)
    padbefore = -min(idx-ctxsize,0)
    end = min(idx+len(word)+ctxsize,len(text))
    passage = text[start:end+1].replace(chr(160)," ")
    if(wrap):
        passage = passage.replace(word,"["+word+"]")
    padafter = (2*ctxsize+len(word)+1)-len(passage)-padbefore
    if(wrap):
        padafter = padafter+2
    paddedpassage = (" "*padbefore) + passage + (" "*padafter)
    return paddedpassage